@@ -33,8 +33,8 @@ alt_name_dict = {
 }  # type: dict
 
 custom_field_tests = {
-    "material_ids": ["mp-22526"],
-    "battery_ids": ["mp-22526_Li"],
+    "material_ids": ["mp-16722"],
+    "battery_ids": ["mp-16722_Al"],
     "working_ion": Element("Li"),
     "formula": "CoO2",
     "chemsys": "Co-O",
