@@ -6,6 +6,7 @@ import importlib
 import numpy as np
 import pytest
 from emmet.core.tasks import TaskDoc
+from emmet.core.thermo import ThermoType
 from emmet.core.vasp.calc_types import CalcType
 from pymatgen.analysis.phase_diagram import PhaseDiagram
 from pymatgen.analysis.pourbaix_diagram import IonEntry, PourbaixDiagram, PourbaixEntry
@@ -18,7 +19,11 @@ from pymatgen.electronic_structure.bandstructure import (
     BandStructureSymmLine,
 )
 from pymatgen.electronic_structure.dos import CompleteDos
-from pymatgen.entries.compatibility import MaterialsProjectAqueousCompatibility
+from pymatgen.entries.compatibility import (
+    MaterialsProjectAqueousCompatibility,
+    MaterialsProject2020Compatibility,
+)
+from pymatgen.entries.mixing_scheme import MaterialsProjectDFTMixingScheme
 from pymatgen.entries.computed_entries import ComputedEntry, GibbsComputedStructureEntry
 from pymatgen.io.cif import CifParser
 from pymatgen.io.vasp import Chgcar
@@ -139,10 +144,7 @@ class TestMPRester:
             assert e.data.get("energy_above_hull", None) is not None
 
         # Conventional structure
-        formula = "BiFeO3"
-        entry = mpr.get_entry_by_material_id(
-            "mp-22526", inc_structure=True, conventional_unit_cell=True
-        )[0]
+        entry = mpr.get_entry_by_material_id("mp-22526", conventional_unit_cell=True)[1]
 
         s = entry.structure
         assert pytest.approx(s.lattice.a) == s.lattice.b
@@ -152,9 +154,7 @@ class TestMPRester:
         assert pytest.approx(s.lattice.gamma) == 120
 
         # Ensure energy per atom is same
-        prim = mpr.get_entry_by_material_id(
-            "mp-22526", inc_structure=True, conventional_unit_cell=False
-        )[0]
+        prim = mpr.get_entry_by_material_id("mp-22526", conventional_unit_cell=False)[1]
 
         s = prim.structure
         assert pytest.approx(s.lattice.a) == s.lattice.b
@@ -327,7 +327,7 @@ class TestMPRester:
                 chunk_size=1000, num_chunks=10, fields=["material_id"]
             )
         ]
-        docs = mpr.summary.search(material_ids=mpids, fields=["material_ids"])
+        docs = mpr.summary.search(material_ids=mpids, fields=["material_id"])
         assert len(docs) == 10000
 
     def test_get_api_key_endpoint_from_env_var(self, monkeypatch: pytest.MonkeyPatch):
@@ -371,3 +371,141 @@ class TestMPRester:
         monkeypatch.setenv("MP_API_KEY", "INVALID")
         with pytest.raises(ValueError, match="Keys for the new API are 32 characters"):
             MPRester().get_structure_by_material_id("mp-149")
+
+    def test_get_cohesive_energy_per_atom_utility(self):
+        composition = {
+            "H": 5,
+            "V": 2,
+            "P": 3,
+        }
+        toten_per_atom = -2.0e3
+        atomic_energies = {"H": -13.6, "V": -7.2, "P": -0.1}
+
+        by_hand_e_coh = toten_per_atom - sum(
+            atomic_energies[k] * v for k, v in composition.items()
+        ) / sum(composition.values())
+
+        assert MPRester._get_cohesive_energy(
+            composition, toten_per_atom, atomic_energies
+        ) == pytest.approx(by_hand_e_coh)
+
+    def test_get_atom_references(self, mpr):
+        ae = mpr.get_atom_reference_data(funcs=("PBE",))
+        assert list(ae) == ["PBE"]
+        assert len(ae["PBE"]) == 89
+        assert all(isinstance(v, float) for v in ae["PBE"].values())
+
+        ae = mpr.get_atom_reference_data()
+        assert set(ae) == {"PBE", "r2SCAN", "SCAN"}
+        assert all(len(entries) == 89 for entries in ae.values())
+        assert all(
+            isinstance(v, float) for entries in ae.values() for v in entries.values()
+        )
+
+    def test_get_cohesive_energy(self):
+        ref_e_coh = {
+            "atom": {
+                "mp-123": -4.029208982500002,
+                "mp-149": -4.669184594999999,
+                "mp-4163": -6.351402620416668,
+                "mp-19017": -4.933409960714286,
+            },
+            "formula_unit": {
+                "mp-123": -4.029208982500002,
+                "mp-149": -4.669184594999999,
+                "mp-4163": -76.21683144500001,
+                "mp-19017": -34.533869725,
+            },
+        }
+        e_coh = {}
+        for monty_decode in (True, False):
+            with MPRester(
+                use_document_model=monty_decode, monty_decode=monty_decode
+            ) as _mpr:
+                for norm, refs in ref_e_coh.items():
+                    _e_coh = _mpr.get_cohesive_energy(list(refs), normalization=norm)
+                    if norm == "atom":
+                        e_coh["serial" if monty_decode else "noserial"] = _e_coh.copy()
+
+                    # Ensure energies match reference data
+                    assert all(v == pytest.approx(refs[k]) for k, v in _e_coh.items())
+
+        # Ensure energies are the same regardless of serialization
+        assert all(
+            v == pytest.approx(e_coh["noserial"][k]) for k, v in e_coh["serial"].items()
+        )
+
+    @pytest.mark.parametrize(
+        "chemsys, thermo_type",
+        [
+            [("Fe", "P"), "GGA_GGA+U"],
+            [("Li", "S"), ThermoType.GGA_GGA_U_R2SCAN],
+            [("Ni", "Se"), ThermoType.R2SCAN],
+            [("Ni", "Kr"), "R2SCAN"],
+        ],
+    )
+    def test_get_stability(self, chemsys, thermo_type):
+        """
+        This test is adapted from the pymatgen one - the scope is broadened
+        to include more diverse chemical environments and thermo types which
+        reflect the scope of the current MP database.
+        """
+        with MPRester() as mpr:
+            entries = mpr.get_entries_in_chemsys(
+                chemsys, additional_criteria={"thermo_types": [thermo_type]}
+            )
+
+            no_compound_entries = all(
+                len(entry.composition.elements) == 1 for entry in entries
+            )
+
+            modified_entries = [
+                ComputedEntry(
+                    entry.composition,
+                    entry.uncorrected_energy + 0.01,
+                    parameters=entry.parameters,
+                    entry_id=f"mod_{entry.entry_id}",
+                )
+                for entry in entries
+                if entry.composition.reduced_formula in ["Fe2P", "".join(chemsys)]
+            ]
+
+            if len(modified_entries) == 0:
+                # create fake entry to get PD retrieval to fail
+                modified_entries = [
+                    ComputedEntry(
+                        "".join(chemsys),
+                        np.average([entry.energy for entry in entries]),
+                        entry_id=f"hypothetical",
+                    )
+                ]
+
+            if no_compound_entries:
+                with pytest.warns(UserWarning, match="No phase diagram data available"):
+                    mpr.get_stability(modified_entries, thermo_type=thermo_type)
+                return
+
+            else:
+                rester_ehulls = mpr.get_stability(
+                    modified_entries, thermo_type=thermo_type
+                )
+
+        all_entries = entries + modified_entries
+
+        compat = None
+        if thermo_type == "GGA_GGA+U":
+            compat = MaterialsProject2020Compatibility()
+        elif thermo_type == "GGA_GGA+U_R2SCAN":
+            compat = MaterialsProjectDFTMixingScheme(run_type_2="r2SCAN")
+
+        if compat:
+            all_entries = compat.process_entries(all_entries)
+
+        pd = PhaseDiagram(all_entries)
+        for entry in all_entries:
+            if str(entry.entry_id).startswith("mod"):
+                for dct in rester_ehulls:
+                    if dct["entry_id"] == entry.entry_id:
+                        data = dct
+                        break
+                assert pd.get_e_above_hull(entry) == pytest.approx(data["e_above_hull"])
